@@ -33,12 +33,8 @@ def crossdomain(f):
             return self._prep_response("CORS request blacklisted", status=405)
         if self.cors_allowed is not None and hostname not in self.cors_allowed:
             return self._prep_response("CORS request refused", status=405)
-        if hostname.strip() == "":
-            # allows requests from phone apps etc
-            h['Access-Control-Allow-Origin'] = "null"
-        else:
-            h['Access-Control-Allow-Origin'] = "%s://%s" % \
-                (request.headers.environ['wsgi.url_scheme'], hostname)
+        if 'origin' in request.headers:
+            h['Access-Control-Allow-Origin'] = request.headers['origin']
 
         # Request header checks
         if 'access-control-request-headers' in request.headers:
